import imread
from . import file_path

def test_detect_format():
    assert imread.detect_format(file_path('GOOD.PNG')) == 'png'
    assert imread.detect_format(file_path('ghsystem_flame.jpg')) == 'jpeg'
    data = open(file_path('ghsystem_flame.jpg'), 'rb').read(16)
    assert imread.detect_format(data, 1) == 'jpeg'

    assert imread.detect_format(b'\xff\xd8\xff\0', 1) == 'jpeg'


def test_supports_format():
    assert imread.supports_format("png")
    assert not imread.supports_format("xyz-format-which-does-not-exist")



from nose.tools import raises
from . import file_path
from imread import imread

def test_with_dot():
    f = imread(file_path('good.png'))
    assert f.shape == (2,2)

def test_uppercase():
    f = imread(file_path('GOOD.PNG'))
    assert f.shape == (2,2)

@raises(ValueError)
def test_no_ext():
    imread('file_without_extension')


def test_formatstr():
    f = imread(file_path('good'), formatstr='png')
    assert f.shape == (2,2)


def test_as_grey():
    im = imread(file_path('star1.bmp'), as_grey=False)
    assert len(im.shape) == 3
    im = imread(file_path('star1.bmp'), as_grey=True)
    assert len(im.shape) == 2



from nose.tools import with_setup, raises
import numpy as np
from imread import imread, imsave
from . import file_path
_filename = 'imread_testing_file.jpg'

def remove_files(filelist):
    def perform_removal():
        from os import unlink
        for f in filelist:
            try:
                unlink(f)
            except:
                pass
    def wrap(f):
        return with_setup(teardown=perform_removal)(f)
    return wrap

@remove_files([_filename])
def test_jpeg():
    f = np.arange(64*16).reshape((64,16))
    f %= 16
    f = f.astype(np.uint8)
    imsave(_filename, f, 'jpeg')
    g = imread(_filename).squeeze()
    assert np.mean(np.abs(f.astype(float)-g)) < 1.


@raises(RuntimeError)
def test_error():
    imread(file_path('error.jpg'))

@raises(OSError)
def test_error_noent():
    imread(file_path('this-file-does-not-exist.jpeg'))



@remove_files(['imread_def.jpg', 'imread_def91.jpg'])
def test_quality():
    def pixel_diff(a):
        return np.mean(np.abs(a.astype(float) - data))

    data = np.arange(256*256*3)
    data %= 51
    data = data.reshape((256,256,3))
    data = data.astype(np.uint8)
    imsave('imread_def.jpg', data)
    imsave('imread_def91.jpg', data, opts={'jpeg:quality': 91} )
    readback    = imread('imread_def.jpg')
    readback91  = imread('imread_def91.jpg')
    assert pixel_diff(readback91) < pixel_diff(readback)



# Copyright (C) 2012-2016, Luis Pedro Coelho <luis@luispedro.org>
# vim: set ts=4 sts=4 sw=4 expandtab smartindent:
#
# License: MIT (see COPYING.MIT file)

from __future__ import print_function
try:
    from .imread import imread, imwrite, imread_multi, imread_from_blob, imwrite_multi
    from .imread import detect_format, supports_format
    from .imread import imload, imsave, imload_multi, imload_from_blob, imsave_multi
    from .imread_version import __version__
except ImportError as e:
    import sys
    print('''\
Could not import submodules (exact error was: {0}).

There are many reasons for this error the most common one is that you have
either not built the packages or have built (using `python setup.py build`) or
installed them (using `python setup.py install`) and then proceeded to test
mahotas-imread **without changing the current directory**.

Try installing and then changing to another directory before importing mahotas.
'''.format(e), file=sys.stderr)




def run(verbose=False):
    import nose
    from os import path
    currentdir = path.dirname(__file__)
    updir = path.join(currentdir, '..')
    argv = ['', '--exe', '-w', updir]
    if verbose:
        argv.append('--verbose')
    nose.run('imread', argv=argv)

def file_path(fname):
    from os import path
    return path.join(
            path.dirname(__file__),
            'data',
            fname)



from nose.tools import with_setup, raises
import numpy as np
from . import file_path
from imread import imread, imsave

_filename = 'imread_testing_file.png'

def _remove_file():
    from os import unlink
    try:
        unlink(_filename)
    except:
        pass

@with_setup(teardown=_remove_file)
def test_png_raw():
    simple = np.arange(16*16).reshape((16,16))
    simple = simple.astype(np.uint8)
    imsave(_filename, simple, 'png')
    back = imread(_filename)
    assert np.all(simple == back)

@with_setup(teardown=_remove_file)
def test_asym():
    simple = np.arange(16*16).reshape((32,8))
    simple = simple.astype(np.uint8)
    imsave(_filename, simple, 'png')
    back = imread(_filename)
    assert np.all(simple == back)

@with_setup(teardown=_remove_file)
def test_random():
    np.random.seed(23)
    for i in range(8):
        simple = np.random.random_sample((64,64,3))
        simple *= 255
        simple = simple.astype(np.uint8)
        if i < 3:
            simple[:,:,i] = 0
        imsave(_filename, simple, 'png')
        back = imread(_filename)
        assert np.all(simple == back)


@with_setup(teardown=_remove_file)
def test_non_carray():
    np.random.seed(87)
    simple = np.random.random_sample((128,128,3))
    simple *= 255
    simple = simple.astype(np.uint8)
    simple = simple[32:64,32::2]
    imsave(_filename, simple, 'png')
    back = imread(_filename)
    assert np.all(simple == back)


def test_binary():
    f = imread(file_path('bit1.png'))
    assert f.dtype == np.bool_

@raises(RuntimeError)
def test_error():
    imread(file_path('error.png'))

def test_regression():
    im = imread(file_path('palette_zero.png'))
    assert im.sum() == 0
    assert im.shape == (128, 64, 3)


def test_16bit():
    f = imread(file_path('arange512_16bit.png'))
    assert np.all(f.ravel() == np.arange(512))


@with_setup(teardown=_remove_file)
def test_write_16bit():
    f = np.arange(100000, dtype=np.uint16)*1000
    f = f.reshape((100,-1))
    imsave(_filename, f)
    f2 = imread(_filename)
    assert np.all(f == f2)

@with_setup(teardown=_remove_file)
def test_write_16bit_rgb():
    f = np.random.random((16,8,3)) * 65535.0
    f = f.astype(np.uint16)
    imsave(_filename, f)
    f2 = imread(_filename)
    assert np.all(f == f2)

def test_strip_alpha():
    w_alpha = imread(file_path('rgba.png'))
    wno_alpha = imread(file_path('rgba.png'), opts={'strip_alpha':True})
    assert np.all(w_alpha[:,:,:3] == wno_alpha)



from nose.tools import with_setup, raises
import numpy as np
from imread import imread, imsave, imread_multi, imsave_multi
from . import file_path

_filename = 'imread_testing_file.tiff'

def _remove_file():
    from os import unlink
    try:
        unlink(_filename)
    except:
        pass

@raises(RuntimeError)
def test_error():
    imread(file_path('error.tif'))


@with_setup(teardown=_remove_file)
def test_read_back():
    simple = np.arange(16*16).reshape((16,16))
    simple = simple.astype(np.uint8)
    imsave(_filename, simple)
    back = imread(_filename)
    assert np.all(simple == back)

@with_setup(teardown=_remove_file)
def test_read_back_16():
    np.random.seed(21)
    simple = np.random.random_sample((128,128))
    simple *= 8192
    simple = simple.astype(np.uint16)
    imsave(_filename, simple)
    back = imread(_filename)
    assert np.all(simple == back)

def test_monochrome():
    mono = imread(file_path('mono.tif'))
    assert mono.shape == (8,8)
    z = np.zeros((8,8),np.uint8)
    z.flat[::3] = 1
    assert np.all(z == mono)


def test_multi():
    assert len(imread_multi(file_path('stack.tiff'))) == 2

@with_setup(teardown=_remove_file)
def test_read_back_with_metadata():
    simple = np.arange(16*16).reshape((16,16))
    simple = simple.astype(np.uint8)
    meta = b'123qwe'
    imsave(_filename, simple, metadata=meta)
    back,meta_read = imread(_filename, return_metadata=True)
    assert np.all(simple == back)
    assert meta == meta_read


@with_setup(teardown=_remove_file)
def test_read_back_colour():
    im = np.arange(256).astype(np.uint8).reshape((32,-1))
    im = np.dstack([im, im*0, 255-im])
    imsave(_filename, im)
    im2 = imread(_filename)
    assert im.shape == im2.shape
    assert np.all(im == im2)

@with_setup(teardown=_remove_file)
def test_read_back_colour_16bit():
    im = np.random.random((16,8,3)) * 65535.0
    im = im.astype(np.uint16)
    imsave(_filename, im)
    im2 = imread(_filename)
    assert im.shape == im2.shape
    assert np.all(im == im2)

@with_setup(teardown=_remove_file)
def test_horizontal_predictor():
    im = imread(file_path('arange512_16bit.png'))
    im2 = im.copy()
    imsave(_filename, im, opts={'tiff:horizontal-predictor': True})
    assert np.all(im == im2)

    im3 = imread(_filename)
    assert np.all(im == im3)

@with_setup(teardown=_remove_file)
def test_imsave_multi():
    im = imread(file_path('arange512_16bit.png'))
    im2 = im[::4, ::4]
    ims = [im, im2]
    imsave_multi(_filename, ims)
    ims2 = imread_multi(_filename)
    assert len(ims) == len(ims2)
    for a,b in zip(ims, ims2):
        assert np.all(a == b)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2013, Luis Pedro Coelho <luis@luispedro.org>
# vim: set ts=4 sts=4 sw=4 expandtab smartindent:
#
# License: MIT (see COPYING.MIT file)

import imread

def readxcf(xcf_filename, formatstr, _flags):
    '''
    im = readxcf(xcf_filename, formatstr)

    Returns a numpy array with the (flattened) XCF file.

    Depends on `xcf2png` being available.

    Parameters
    ----------
    xcf_filename : str
        Filename

    formatstr : str
        format. Must be 'xcf'

    Returns
    -------
    im : ndarray
    '''
    from os import system, unlink
    from tempfile import NamedTemporaryFile
    if formatstr != 'xcf':
        raise ValueError('imread.imread.readxcf: Format string must be \'xcf\'')

    N = NamedTemporaryFile(suffix='.png')
    output = system('xcf2png %s >%s' % (xcf_filename,N.name))
    if output:
        raise OSError('imread.readxcf: xcf format is only supported through the xcf2png utility, which imread could not run')
    return imread.imread(N.name, return_metadata=True)

special = {
    'xcf' : readxcf,
}



# Copyright: Luis Pedro Coelho <luis@luispedro.org>, 2012-2018
# License: MIT
import numpy as np

def read_roi(fileobj):
    '''
    points = read_roi(fileobj)

    Read ImageJ's ROI format

    Parameters
    ----------
    fileobj: should be a file-like object

    Returns
    -------

    points: a list of points
    '''
# This is based on:
# http://rsbweb.nih.gov/ij/developer/source/ij/io/RoiDecoder.java.html
# http://rsbweb.nih.gov/ij/developer/source/ij/io/RoiEncoder.java.html


    SPLINE_FIT = 1
    DOUBLE_HEADED = 2
    OUTLINE = 4
    OVERLAY_LABELS = 8
    OVERLAY_NAMES = 16
    OVERLAY_BACKGROUNDS = 32
    OVERLAY_BOLD = 64
    SUB_PIXEL_RESOLUTION = 128
    DRAW_OFFSET = 256


    pos = [4]
    def get8():
        pos[0] += 1
        s = fileobj.read(1)
        if not s:
            raise IOError('readroi: Unexpected EOF')
        return ord(s)

    def get16():
        b0 = get8()
        b1 = get8()
        return (b0 << 8) | b1

    def get32():
        s0 = get16()
        s1 = get16()
        return (s0 << 16) | s1

    def getfloat():
        v = np.int32(get32())
        return v.view(np.float32)

    magic = fileobj.read(4)
    if magic != b'Iout':
        raise IOError('Magic number not found')
    version = get16()

    # It seems that the roi type field occupies 2 Bytes, but only one is used
    roi_type = get8()
    # Discard second Byte:
    get8()

    if not (0 <= roi_type < 11):
        raise ValueError('roireader: ROI type %s not supported' % roi_type)

    if roi_type != 7:
        raise ValueError('roireader: ROI type %s not supported (!= 7)' % roi_type)

    top = get16()
    left = get16()
    bottom = get16()
    right = get16()
    n_coordinates = get16()

    x1 = getfloat() 
    y1 = getfloat() 
    x2 = getfloat() 
    y2 = getfloat()
    stroke_width = get16()
    shape_roi_size = get32()
    stroke_color = get32()
    fill_color = get32()
    subtype = get16()
    if subtype != 0:
        raise ValueError('roireader: ROI subtype {} not supported (!= 0)'.format(subtype))
    options = get16()
    arrow_style = get8()
    arrow_head_size = get8()
    rect_arc_size = get16()
    position = get32()
    header2offset = get32()

    if options & SUB_PIXEL_RESOLUTION:
        getc = getfloat
        points = np.empty((n_coordinates, 2), dtype=np.float32)
    else:
        getc = get16
        points = np.empty((n_coordinates, 2), dtype=np.int16)
    points[:,1] = [getc() for i in range(n_coordinates)]
    points[:,0] = [getc() for i in range(n_coordinates)]
    points[:,1] += left
    points[:,0] += top
    points -= 1
    return points

def read_roi_zip(fname):
    '''
    Reads all ROIs in a ZIP file

    Parameters
    ----------
    fname : str
        Input filename

    Returns
    -------
    rois: list of ROIs
        Each ROI is a vector of 2D points

    See Also
    --------
    read_roi: function, reads a single ROI
    '''
    import zipfile
    with zipfile.ZipFile(fname) as zf:
        return [read_roi(zf.open(n))
                    for n in zf.namelist()]



import sys 
from imread import imread
from . import file_path
from nose import SkipTest

def has_xcf2png():
    # there is no native xcf2png utility for Windows
    if sys.platform.startswith('win'):
        return False
    try:
        from shutil import which
    except:
        from distutils.spawn import find_executable as which

    c = which('xcf2png')
    return (c is not None)


def test_xcf():
    if not has_xcf2png():
        raise SkipTest
    im = imread(file_path('diag.xcf'))
    assert im.shape == (8, 8, 3)
    assert im.max(2).diagonal().sum() == 0



# Copyright (C) 2012-2016, Luis Pedro Coelho <luis@luispedro.org>
# vim: set ts=4 sts=4 sw=4 expandtab smartindent:
#
# License: MIT (see COPYING.MIT file)

import numpy as np

from . import _imread

from .special import special

def _parse_formatstr(filename, formatstr, funcname):
    if formatstr is not None:
        return formatstr
    from os import path
    _,ext = path.splitext(filename)
    if len(ext) and ext[0] == '.':
        return ext[1:].lower()
    raise ValueError('imread.{0}: Could not identify format from filename: `{1}`'.format(funcname,filename))

def _as_grey(im, as_grey):
    if not as_grey or len(im.shape) != 3: return im
    if im.shape[2] == 1:
        return im.squeeze()
    # these are the values that wikipedia says are typical
    transform = np.array([ 0.30,  0.59,  0.11])
    return np.dot(im, transform)

def _norm_path(path):
    import os
    if hasattr(os, 'PathLike') and isinstance(path, os.PathLike):
        return str(path)
    return path

def imread(filename, as_grey=False, formatstr=None, return_metadata=False, opts=None):
    '''
    im = imread(filename, as_grey=False, formatstr={filename extension}, return_metadata=False)
    im,meta = imread(filename, as_grey=False, formatstr={filename extension}, return_metadata=True)

    Read an image into a ndarray from a file.

    Parameters
    ----------
    filename : str
        filename
    as_grey : boolean, optional
        Whether to convert to grey scale image (default: no)
    formatstr : str, optional
        Format name. This is typically the same as the extension of the file
        and inferred from there. However, if you have a file whose extension
        does not correspond to the format, you can pass it explicitly.
    return_metadata : bool, optional
        Whether to return metadata (default: False)
    opts : dict, optional
        Other options. Support will depend on the backend. Currently used

            strip_alpha: bool
                Whether to strip the alpha channel.

    Returns
    -------
    im : ndarray
        The type of this array will depend on the contents of the file and of
        `as_grey`. Conversion from colour to grayscale will return a floating
        point image.
    meta: str
        Metadata. The metadata in the file Only returned if ``return_metadata``.

    See Also
    --------
    imread_from_blob : function
        Read from a in-memory string
    '''
    if opts is None:
        opts = {}
    filename = _norm_path(filename)
    formatstr = _parse_formatstr(filename, formatstr, 'imread')
    reader = special.get(formatstr, _imread.imread)
    imdata,meta = reader(filename, formatstr, opts)
    imdata = _as_grey(imdata, as_grey)
    if return_metadata:
        return imdata, meta
    return imdata

imload = imread


def imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata=False, opts=None):
    '''
    imdata = imread_from_blob(blob, formatstr=None, as_grey=False, return_metadata={True})
    imdata,metadata = imread_from_blob(blob, formatstr=None, as_grey={False}, return_metadata=True)

    Read an image into a ndarray from an in-memory blob.

    Note that the parameter order is changed wrt. ``imread`` because
    **formatstr** is a mandatory parameter to this function!

    Parameters
    ----------
    blob : str (bytes in Py3)
        input data
    formatstr : str, optional
        Format name. This is the file extension typically associated with this
        format. If None is given, then ``detect_format`` is used to auto-detect
        the format.
    as_grey : boolean, optional
        Whether to convert to grey scale image (default: no)
    return_metadata : bool, optional
        Whether to return metadata (default: False)
    opts : dict, optional
        Other options. Support will depend on the backend. Currently used

            strip_alpha: bool
                Whether to strip the alpha channel.

    Returns
    -------
    im : ndarray
        The type of this array will depend on the contents of the file and of
        `as_grey`. Conversion from colour to grayscale will return a floating
        point image.
    meta: str
        Metadata. The metadata in the file Only returned if ``return_metadata``.

    See Also
    --------
    imread : function
        Read from a file on disk
    '''
    if opts is None:
        opts = {}
    reader = _imread.imread_from_blob
    if formatstr is None:
        formatstr = detect_format(blob, is_blob=True)
    imdata,meta = reader(blob, formatstr, opts)
    imdata = _as_grey(imdata, as_grey)
    if return_metadata:
        return imdata,meta
    return imdata

imload_from_blob = imread_from_blob

def imread_multi(filename, formatstr=None):
    '''
    images = imread_multi(filename, formatstr={from filename})

    The file type is guessed from `filename`.

    Parameters
    ----------
    filename : str
        filename

    formatstr : str, optional
        file format. If ``None``, then it is derived from the filename.

    Returns
    -------
    images : list
    '''
    filename = _norm_path(filename)
    formatstr = _parse_formatstr(filename, formatstr, 'imread')
    return _imread.imread_multi(filename, formatstr, '')

imload_multi = imread_multi


def imsave(filename, array, formatstr=None, metadata=None, opts=None):
    '''
    imsave(filename, array, formatstr={auto-detect}, metadata={None}, opts={})

    Writes `array` into file `filename`

    Parameters
    ----------
    filename : str
        path on file system
    array : ndarray-like
    formatstr: str, optional
        format string
    metadata: bytes, optional
        metadata to write to file. Note that not all formats support writing
        metadata.
    opts: dict, optional
        This is a dictionary of options. Any non-applicable option is typically
        silently ignored. Currently, the following options are accepted:

        jpeg:quality
            An integer 1-100 determining the quality used by JPEG backend
            (default is libjpeg default: 75).

        tiff:compress
            Whether to use compression when saving TIFF (default: True)

        tiff:horizontal-predictor
            Whether to use horizontal prediction in TIFF. This defaults to True
            for 16 bit images, and to False for 8 bit images. This is because
            compressing 16 bit images without horizontal prediction is often
            counter-productive (see http://www.asmail.be/msg0055176395.html)


        png:compression_level
            Compression level to use, from 0 (no compression) to 9. Setting
            this parameter to 0 is discouraged as setting it to 1 already
            provides a modicum of compression at no extra computational cost.

    '''
    if opts is None:
        opts = {}
    if not np.issubdtype(array.dtype, np.integer):
        raise TypeError('imread.imsave: only integer images are supported')
    array = np.ascontiguousarray(array)
    filename = _norm_path(filename)
    if formatstr is None:
        dot = filename.rfind('.')
        if dot < 0:
            raise ValueError('imread.imsave: dot not found in filename ({})'.format(filename))
        formatstr = filename[dot+1:]
    if metadata is not None:
        opts['metadata'] = metadata
    _imread.imsave(filename, formatstr, array, opts)

def imsave_multi(filename, arrays, formatstr=None, opts=None):
    '''Saves multiple arrays into a single file

    Only certain formats (in the current version of imread, only TIFF) support
    writing multiple outputs.

    Parameters
    ----------
    filename : str
        path on file system
    arrays : list of ndarray-like
    formatstr: str, optional
        format string
    metadata: bytes, optional
        metadata to write to file. Note that not all formats support writing
        metadata.
    opts: dict, optional
        This is a dictionary of options. Any non-applicable option is typically
        silently ignored. Currently, the following options are accepted:

        jpeg:quality
            An integer 1-100 determining the quality used by JPEG backend
            (default is libjpeg default: 75).

        tiff:compress
            Whether to use compression when saving TIFF (default: True)

        tiff:horizontal-predictor
            Whether to use horizontal prediction in TIFF. This defaults to True
            for 16 bit images, and to False for 8 bit images. This is because
            compressing 16 bit images without horizontal prediction is often
            counter-productive (see http://www.asmail.be/msg0055176395.html)


        png:compression_level
            Compression level to use, from 0 (no compression) to 9. Setting
            this parameter to 0 is discouraged as setting it to 1 already
            provides a modicum of compression at no extra computational cost.
    '''
    if opts is None:
        opts = {}
    for array in arrays:
        if not np.issubdtype(array.dtype, np.integer):
            raise TypeError('imread.imsave: only integer images are supported')
    arrays = [np.ascontiguousarray(array) for array in arrays]
    filename = _norm_path(filename)
    if formatstr is None:
        dot = filename.rfind('.')
        if dot < 0:
            raise ValueError('imread.imsave_multi: dot not found in filename ({})'.format(filename))
        formatstr = filename[dot+1:]
    _imread.imsave_multi(filename, formatstr, arrays, opts)

imwrite = imsave
imwrite_multi = imsave_multi


def detect_format(filename_or_blob, is_blob=False):
    '''
    Detect format using magic numbers

    Note that this function does not perform any checks that the data is OK,
    just checks magic numbers. Not all formats can be autodetected.

    Parameters
    ----------
    filename_or_blob : bytes (str in Python 2)
    is_blob : boolean, optional
        If true, then filename_or_blob is interpreted as a blob; otherwise,

    Returns
    -------
    format : str or None
        Representation of the format if detected. When no format matches, None
        is returned.
    '''
    if not is_blob:
        filename_or_blob = _norm_path(filename_or_blob)
    return _imread.detect_format(filename_or_blob, bool(is_blob))


def supports_format(formatstr):
    '''Query whether the format is supported by imread

    Parameters
    ----------
    formatstr : str
        format string (for example, 'png')

    Returns
    -------
    is_supported : bool
    '''
    return _imread.supports_format(formatstr)



from nose.tools import with_setup, raises
import numpy as np
from imread import imread
from . import file_path

def test_read():
    im = imread(file_path('star1.bmp'))
    assert np.any(im)
    assert im.shape == (128, 128, 3)

def test_indexed():
    im = imread(file_path('py-installer-indexed.bmp'))
    assert np.any(im)
    assert im.shape == (352, 162, 3)
    assert np.any(im[:,:,0])
    assert np.any(im[:,:,1])
    assert np.any(im[:,:,2])



from imread import imsave
import numpy as np
from nose.tools import raises

@raises(Exception)
def test_non_existing():
    # in 0.2.5 this led to a hard crash!
    arr = np.arange(64,dtype=np.uint8).reshape((8,8))
    imsave('/tmp/test-me.png', arr, 'some format which does not exist')


@raises(TypeError)
def test_bad_args():
    arr = np.arange(64,dtype=np.uint8).reshape((8,8))
    imsave('/tmp/test-me.png', arr, arr)


@raises(TypeError)
def test_save_float():
    im = (np.arange(64*64).reshape((64,64)) % 32 ) * 2.
    imsave('test.jpeg', im)



import imread
from imread.imread import imread_from_blob
import numpy as np
def test_imread_from_blob():
    def compare_with_blob(filename, formatstr):
        from os import path
        filename = path.join(
                        path.dirname(__file__),
                        'data',
                        filename)
        fromfile= imread.imread(filename)
        fromblob = imread_from_blob(open(filename, 'rb').read(), formatstr)
        assert np.all(fromblob == fromfile)
    yield compare_with_blob, 'good.png', 'png'
    yield compare_with_blob, 'good.png', None
    yield compare_with_blob, 'GOOD.PNG', 'png'
    yield compare_with_blob, 'mono.tif', 'tif'
    yield compare_with_blob, 'mono.tif', 'tiff'
    yield compare_with_blob, 'py-installer-indexed.bmp', 'bmp'



import numpy as np
from imread import ijrois

from . import file_path


def test_rois_smoke():
    rois = ijrois.read_roi_zip('./imread/tests/data/rois.zip')
    assert len(rois) == 4
    r = ijrois.read_roi(open('./imread/tests/data/0186-0099.roi', 'rb'))
    assert any([np.array_equal(ri, r) for ri in rois])
